@@ -5,15 +5,26 @@
  */
 
 import esbuild from 'esbuild';
-import path from 'path';
-import { fileURLToPath } from 'url';
-import { createRequire } from 'module';
+import path from 'node:path';
+import { fileURLToPath } from 'node:url';
+import { createRequire } from 'node:module';
+import { writeFileSync } from 'node:fs';
 
 const __filename = fileURLToPath(import.meta.url);
 const __dirname = path.dirname(__filename);
 const require = createRequire(import.meta.url);
 const pkg = require(path.resolve(__dirname, 'package.json'));
 
+const external = [
+  '@lydell/node-pty',
+  'node-pty',
+  '@lydell/node-pty-darwin-arm64',
+  '@lydell/node-pty-darwin-x64',
+  '@lydell/node-pty-linux-x64',
+  '@lydell/node-pty-win32-arm64',
+  '@lydell/node-pty-win32-x64',
+];
+
 esbuild
   .build({
     entryPoints: ['packages/cli/index.ts'],
@@ -21,7 +32,7 @@ esbuild
     outfile: 'bundle/gemini.js',
     platform: 'node',
     format: 'esm',
-    external: [],
+    external,
     alias: {
       'is-in-ci': path.resolve(
         __dirname,
@@ -34,5 +45,13 @@ esbuild
     banner: {
       js: `import { createRequire } from 'module'; const require = createRequire(import.meta.url); globalThis.__filename = require('url').fileURLToPath(import.meta.url); globalThis.__dirname = require('path').dirname(globalThis.__filename);`,
     },
+    loader: { '.node': 'file' },
+    metafile: true,
+    write: true,
+  })
+  .then(({ metafile }) => {
+    if (process.env.DEV === 'true') {
+      writeFileSync('./bundle/esbuild.json', JSON.stringify(metafile, null, 2));
+    }
   })
   .catch(() => process.exit(1));
