@@ -15,23 +15,25 @@ import {
   DiscoveredMCPTool,
 } from '@google/gemini-cli-core';
 
-import { MessageActionReturn } from './types.js';
-import { Type, CallableTool } from '@google/genai';
+import type { MessageActionReturn } from './types.js';
+import type { CallableTool } from '@google/genai';
+import { Type } from '@google/genai';
 
 vi.mock('@google/gemini-cli-core', async (importOriginal) => {
   const actual =
     await importOriginal<typeof import('@google/gemini-cli-core')>();
+  const mockAuthenticate = vi.fn();
   return {
     ...actual,
     getMCPServerStatus: vi.fn(),
     getMCPDiscoveryState: vi.fn(),
-    MCPOAuthProvider: {
-      authenticate: vi.fn(),
-    },
-    MCPOAuthTokenStorage: {
+    MCPOAuthProvider: vi.fn(() => ({
+      authenticate: mockAuthenticate,
+    })),
+    MCPOAuthTokenStorage: vi.fn(() => ({
       getToken: vi.fn(),
       isTokenExpired: vi.fn(),
-    },
+    })),
   };
 });
 
@@ -73,7 +75,7 @@ describe('mcpCommand', () => {
     vi.clearAllMocks();
 
     // Set up default mock environment
-    delete process.env.SANDBOX;
+    vi.unstubAllEnvs();
 
     // Default mock implementations
     vi.mocked(getMCPServerStatus).mockReturnValue(MCPServerStatus.CONNECTED);
@@ -83,7 +85,7 @@ describe('mcpCommand', () => {
 
     // Create mock config with all necessary methods
     mockConfig = {
-      getToolRegistry: vi.fn().mockResolvedValue({
+      getToolRegistry: vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue([]),
       }),
       getMcpServers: vi.fn().mockReturnValue({}),
@@ -119,7 +121,7 @@ describe('mcpCommand', () => {
     });
 
     it('should show an error if tool registry is not available', async () => {
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue(undefined);
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue(undefined);
 
       const result = await mcpCommand.action!(mockContext, '');
 
@@ -133,7 +135,7 @@ describe('mcpCommand', () => {
 
   describe('no MCP servers configured', () => {
     beforeEach(() => {
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue([]),
       });
       mockConfig.getMcpServers = vi.fn().mockReturnValue({});
@@ -184,7 +186,7 @@ describe('mcpCommand', () => {
         ...mockServer3Tools,
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(allTools),
       });
 
@@ -243,7 +245,7 @@ describe('mcpCommand', () => {
         createMockMCPTool('tool2', 'server1', 'This is tool 2 description'),
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -296,7 +298,7 @@ describe('mcpCommand', () => {
         createMockMCPTool('tool1', 'server1', 'This is tool 1 description'),
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -340,7 +342,7 @@ describe('mcpCommand', () => {
       // Mock tools - only server1 has tools
       const mockServerTools = [createMockMCPTool('server1_tool1', 'server1')];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -386,7 +388,7 @@ describe('mcpCommand', () => {
         createMockMCPTool('server2_tool1', 'server2'),
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -523,7 +525,7 @@ describe('mcpCommand', () => {
 
       const mockServerTools = [tool1, tool2];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -566,7 +568,7 @@ describe('mcpCommand', () => {
         createMockMCPTool('tool1', 'server1', 'Tool without schema'),
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
 
@@ -603,7 +605,7 @@ describe('mcpCommand', () => {
         createMockMCPTool('tool1', 'server1', 'Test tool'),
       ];
 
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue(mockServerTools),
       });
     });
@@ -766,7 +768,7 @@ describe('mcpCommand', () => {
       };
 
       mockConfig.getMcpServers = vi.fn().mockReturnValue(mockMcpServers);
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue([]),
       });
 
@@ -787,7 +789,7 @@ describe('mcpCommand', () => {
       };
 
       mockConfig.getMcpServers = vi.fn().mockReturnValue(mockMcpServers);
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue({
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue({
         getAllTools: vi.fn().mockReturnValue([]),
       });
 
@@ -879,7 +881,7 @@ describe('mcpCommand', () => {
                 oauth: { enabled: true },
               },
             }),
-            getToolRegistry: vi.fn().mockResolvedValue(mockToolRegistry),
+            getToolRegistry: vi.fn().mockReturnValue(mockToolRegistry),
             getGeminiClient: vi.fn().mockReturnValue(mockGeminiClient),
             getPromptRegistry: vi.fn().mockResolvedValue({
               removePromptsByServer: vi.fn(),
@@ -891,13 +893,14 @@ describe('mcpCommand', () => {
       context.ui.reloadCommands = vi.fn();
 
       const { MCPOAuthProvider } = await import('@google/gemini-cli-core');
+      const mockAuthProvider = new MCPOAuthProvider();
 
       const authCommand = mcpCommand.subCommands?.find(
         (cmd) => cmd.name === 'auth',
       );
       const result = await authCommand!.action!(context, 'test-server');
 
-      expect(MCPOAuthProvider.authenticate).toHaveBeenCalledWith(
+      expect(mockAuthProvider.authenticate).toHaveBeenCalledWith(
         'test-server',
         { enabled: true },
         'http://localhost:3000',
@@ -927,9 +930,10 @@ describe('mcpCommand', () => {
       });
 
       const { MCPOAuthProvider } = await import('@google/gemini-cli-core');
-      (
-        MCPOAuthProvider.authenticate as ReturnType<typeof vi.fn>
-      ).mockRejectedValue(new Error('Auth failed'));
+      const mockAuthProvider = new MCPOAuthProvider();
+      vi.mocked(mockAuthProvider.authenticate).mockRejectedValue(
+        new Error('Auth failed'),
+      );
 
       const authCommand = mcpCommand.subCommands?.find(
         (cmd) => cmd.name === 'auth',
@@ -972,6 +976,7 @@ describe('mcpCommand', () => {
     it('should refresh the list of tools and display the status', async () => {
       const mockToolRegistry = {
         discoverMcpTools: vi.fn(),
+        restartMcpServers: vi.fn(),
         getAllTools: vi.fn().mockReturnValue([]),
       };
       const mockGeminiClient = {
@@ -983,7 +988,7 @@ describe('mcpCommand', () => {
           config: {
             getMcpServers: vi.fn().mockReturnValue({ server1: {} }),
             getBlockedMcpServers: vi.fn().mockReturnValue([]),
-            getToolRegistry: vi.fn().mockResolvedValue(mockToolRegistry),
+            getToolRegistry: vi.fn().mockReturnValue(mockToolRegistry),
             getGeminiClient: vi.fn().mockReturnValue(mockGeminiClient),
             getPromptRegistry: vi.fn().mockResolvedValue({
               getPromptsByServer: vi.fn().mockReturnValue([]),
@@ -1004,11 +1009,11 @@ describe('mcpCommand', () => {
       expect(context.ui.addItem).toHaveBeenCalledWith(
         {
           type: 'info',
-          text: 'Refreshing MCP servers and tools...',
+          text: 'Restarting MCP servers...',
         },
         expect.any(Number),
       );
-      expect(mockToolRegistry.discoverMcpTools).toHaveBeenCalled();
+      expect(mockToolRegistry.restartMcpServers).toHaveBeenCalled();
       expect(mockGeminiClient.setTools).toHaveBeenCalled();
       expect(context.ui.reloadCommands).toHaveBeenCalledTimes(1);
 
@@ -1039,7 +1044,7 @@ describe('mcpCommand', () => {
     });
 
     it('should show an error if tool registry is not available', async () => {
-      mockConfig.getToolRegistry = vi.fn().mockResolvedValue(undefined);
+      mockConfig.getToolRegistry = vi.fn().mockReturnValue(undefined);
 
       const refreshCommand = mcpCommand.subCommands?.find(
         (cmd) => cmd.name === 'refresh',
