@@ -18,6 +18,11 @@ export {
   shutdownTelemetry,
   isTelemetrySdkInitialized,
 } from './sdk.js';
+export {
+  GcpTraceExporter,
+  GcpMetricExporter,
+  GcpLogExporter,
+} from './gcp-exporters.js';
 export {
   logCliConfiguration,
   logUserPrompt,
@@ -27,23 +32,33 @@ export {
   logApiResponse,
   logFlashFallback,
   logSlashCommand,
+  logConversationFinishedEvent,
   logKittySequenceOverflow,
+  logChatCompression,
+  logToolOutputTruncated,
+  logExtensionEnable,
+  logExtensionInstallEvent,
+  logExtensionUninstall,
 } from './loggers.js';
+export type { SlashCommandEvent, ChatCompressionEvent } from './types.js';
 export {
-  StartSessionEvent,
+  SlashCommandStatus,
   EndSessionEvent,
   UserPromptEvent,
-  ToolCallEvent,
   ApiRequestEvent,
   ApiErrorEvent,
   ApiResponseEvent,
-  TelemetryEvent,
   FlashFallbackEvent,
+  StartSessionEvent,
+  ToolCallEvent,
+  ConversationFinishedEvent,
   KittySequenceOverflowEvent,
-  SlashCommandEvent,
-  makeSlashCommandEvent,
-  SlashCommandStatus,
+  ToolOutputTruncatedEvent,
 } from './types.js';
+export { makeSlashCommandEvent, makeChatCompressionEvent } from './types.js';
+export type { TelemetryEvent } from './types.js';
 export { SpanStatusCode, ValueType } from '@opentelemetry/api';
 export { SemanticAttributes } from '@opentelemetry/semantic-conventions';
 export * from './uiTelemetry.js';
+export { HighWaterMarkTracker } from './high-water-mark-tracker.js';
+export { RateLimiter } from './rate-limiter.js';
