@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { EventEmitter } from 'events';
+import { EventEmitter } from 'node:events';
 import {
   EVENT_API_ERROR,
   EVENT_API_RESPONSE,
@@ -12,7 +12,11 @@ import {
 } from './constants.js';
 
 import { ToolCallDecision } from './tool-call-decision.js';
-import { ApiErrorEvent, ApiResponseEvent, ToolCallEvent } from './types.js';
+import type {
+  ApiErrorEvent,
+  ApiResponseEvent,
+  ToolCallEvent,
+} from './types.js';
 
 export type UiEvent =
   | (ApiResponseEvent & { 'event.name': typeof EVENT_API_RESPONSE })
@@ -63,6 +67,10 @@ export interface SessionMetrics {
     };
     byName: Record<string, ToolCallStats>;
   };
+  files: {
+    totalLinesAdded: number;
+    totalLinesRemoved: number;
+  };
 }
 
 const createInitialModelMetrics = (): ModelMetrics => ({
@@ -96,6 +104,10 @@ const createInitialMetrics = (): SessionMetrics => ({
     },
     byName: {},
   },
+  files: {
+    totalLinesAdded: 0,
+    totalLinesRemoved: 0,
+  },
 });
 
 export class UiTelemetryService extends EventEmitter {
@@ -171,7 +183,7 @@ export class UiTelemetryService extends EventEmitter {
   }
 
   private processToolCall(event: ToolCallEvent) {
-    const { tools } = this.#metrics;
+    const { tools, files } = this.#metrics;
     tools.totalCalls++;
     tools.totalDurationMs += event.duration_ms;
 
@@ -209,6 +221,16 @@ export class UiTelemetryService extends EventEmitter {
       tools.totalDecisions[event.decision]++;
       toolStats.decisions[event.decision]++;
     }
+
+    // Aggregate line count data from metadata
+    if (event.metadata) {
+      if (event.metadata['model_added_lines'] !== undefined) {
+        files.totalLinesAdded += event.metadata['model_added_lines'];
+      }
+      if (event.metadata['model_removed_lines'] !== undefined) {
+        files.totalLinesRemoved += event.metadata['model_removed_lines'];
+      }
+    }
   }
 }
 
