@@ -5,10 +5,14 @@
  */
 
 import AjvPkg from 'ajv';
+import * as addFormats from 'ajv-formats';
 // Ajv's ESM/CJS interop: use 'any' for compatibility as recommended by Ajv docs
 // eslint-disable-next-line @typescript-eslint/no-explicit-any
 const AjvClass = (AjvPkg as any).default || AjvPkg;
 const ajValidator = new AjvClass();
+// eslint-disable-next-line @typescript-eslint/no-explicit-any
+const addFormatsFunc = (addFormats as any).default || addFormats;
+addFormatsFunc(ajValidator);
 
 /**
  * Simple utility to validate objects against JSON Schemas
